#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>
#include <float.h>
#include "iio.h"
#include "multidirectional_linear_voting.h"

typedef struct image_int_s
{
  int * data;
  unsigned int xsize,ysize;
} * image_int;

typedef struct image_float_s
{
  float * data;
  unsigned int xsize,ysize;
} * image_float;


/** Fatal error, print a message to standard-error output and exit.*/
 
static void fatal_error(char * msg)
{
  fprintf(stderr,"Error: %s\n",msg);
  exit(EXIT_FAILURE);
}
/*----------------------------------------------------------------------------*/
/** float image data type

    The pixel value at (x,y) is accessed by:

      image->data[ x + y * image->xsize ]

    with x and y integer.
 */


/*----------------------------------------------------------------------------*/
/** Create a new image_int of size 'xsize' times 'ysize'.
 */
static image_int new_image_int(unsigned int xsize, unsigned int ysize)
{
  image_int image;

  /* check parameters */
  if( xsize == 0 || ysize == 0 ) fatal_error("new_image_int: invalid image size.");

  /* get memory */
  image = (image_int) malloc( sizeof(struct image_int_s) );
  if( image == NULL ) fatal_error("not enough memory.");
  image->data = (int *) calloc( (size_t) (xsize*ysize), sizeof(int) );
  if( image->data == NULL ) fatal_error("not enough memory.");

  /* set image size */
  image->xsize = xsize;
  image->ysize = ysize;

  return image;
}

/*----------------------------------------------------------------------------*/
/** Create a new image_int of size 'xsize' times 'ysize',
    initialized to the value 'fill_value'.
 */
static image_int new_image_int_ini( unsigned int xsize, unsigned int ysize,
                                    int fill_value )
{
  image_int image = new_image_int(xsize,ysize); /* create image */
  unsigned int N = xsize*ysize;
  unsigned int i;

  /* initialize */
  for(i=0; i<N; i++) image->data[i] = fill_value;

  return image;
}

int read_image_int(const char *fname, int **f, int *w, int *h)
{
        float *f_float;
    
	f_float = iio_read_image_float(fname, w, h);
        *f = (int *)calloc((*w * *h),(unsigned int)sizeof(int));   
        for(int i = 0; i < *w * *h; i++)
	   (*f)[i] = (int)  f_float[i];
        free((float *) f_float);
	return *f ? 0 : -1;
}

/*----------------------------------------------------------------------------*/
/** Free memory used in image_int 'i'.
 */
static void free_image_int(image_int i)
{
  if( i == NULL || i->data == NULL )
    fatal_error("free_image_int: invalid input image.");
  free( (void *) i->data );
  free( (void *) i );
}



/*----------------------------------------------------------------------------*/
/** Create a new image_float of size 'xsize' times 'ysize'.
 */
static image_float new_image_float(unsigned int xsize, unsigned int ysize)
{
  image_float image;

  /* check parameters */
  if( xsize == 0 || ysize == 0 ) fatal_error("new_image_float: invalid image size.");

  /* get memory */
  image = (image_float) malloc( sizeof(struct image_float_s) );
  if( image == NULL ) fatal_error("not enough memory.");
  image->data = (float *) calloc( (size_t) (xsize*ysize), sizeof(float) );
  if( image->data == NULL ) fatal_error("not enough memory.");

  /* set image size */
  image->xsize = xsize;
  image->ysize = ysize;

  return image;
}
float read_image_float(const char *fname, float **f, int *w, int *h)
{
        float *f_float;
	f_float = iio_read_image_float(fname, w, h);
        *f = (float *)calloc((*w * *h),(unsigned int)sizeof(float));   
        for(int i = 0; i < *w * *h; i++)
	   (*f)[i] =   f_float[i];

	return *f ? 0 : -1;
}




/*----------------------------------------------------------------------------*/
/** Free memory used in image_float 'i'.
 */
static void free_image_float(image_float i)
{
  if( i == NULL || i->data == NULL )
    fatal_error("free_image_float: invalid input image.");
  free( (void *) i->data );
  free( (void *) i );
}


void putbrd_int(image_int f, int v)
 {
  register int *temp;
 register int *s,*d;
 register int i,j;
int nx,ny;

f->xsize += 2;
f->ysize += 2;
nx = f->xsize; ny = f->ysize;
     temp = f->data;
     f->data = (int *)calloc((nx*ny),(unsigned int)sizeof(int)); 
     d = f->data;
     for(i = 0;i < nx;i++) *d++ = v;
     s = temp;
     for(j = 1;j < ny-1;j++){
         *d++ = v;
         for(i = 1;i != nx-1;i++) *d++ = *s++; 
         *d++ = v;
     }
     for(i = 0;i <nx;i++) *d++ = v;
     free((char *)temp); 
 }


void 
rmbrd_int ( /*Removes borders.*/
    image_int f
)
{
register int *temp,*s,*d;
register int i,j;
int nx,ny;

f->xsize -= 2;
f->ysize -= 2;
nx = f->xsize; ny = f->ysize;

    temp = f->data;
    f->data = (int *)calloc((nx*ny),(unsigned int)sizeof(int)); 
    d = f->data;
    s = temp+(nx)+3;
    for(j = 0;j != (ny);j++){
        for(i = 0;i != nx;i++) *d++ = *s++;
        s += 2;
    }
    free((char *)temp);
}



void compute_cc(int *bin,image_int app, int x,int y,int labelnum)
{

register int *nbs;
int **nxt,**nxte;
int *d,*da,*temp;
int **bq,**eq;
int  value;
int offst[8],dir;
image_int ima;
int nx = app->xsize, ny = app->ysize;

//allocate memory for the image 
ima = new_image_int(nx,ny);
	    for (int i = 0; i < nx *ny; i++) {
              ima->data[i]= bin[i];
	    }

putbrd_int(ima,(int)BIGVAL);

putbrd_int(app,(int)BIGVAL);
nx = app->xsize; ny = app->ysize;
offst[0] = 1; offst[1] = 1 - nx; offst[2] = -nx;
offst[3] = -nx - 1; offst[4] = -1; offst[5] = nx - 1;
offst[6] = nx; offst[7] = 1 + nx;
nxt = bq = eq = (int **) malloc((unsigned int)sizeof(int *)*8*((nx)*(ny)));
nxte = nxt + 8*(nx*ny) - 1;

d = ima->data +  (y+1)*(nx)+(x+1);
da = app->data +  (y+1)*(nx)+(x+1);

   value = *d;
   *d = *da = labelnum;
   *eq = d;
   eq++; if(eq >= nxte) eq = nxt; /*Adding point to queue.*/
   while(bq != eq){
   	d = *bq;
   	bq++; if (bq >= nxte) bq = nxt;
   	/*Checks neigbours and append them to the queue.*/
	for(dir=0;dir<8;dir++){
		nbs = d + offst[dir];
		if(*nbs==value ){
			*eq = nbs;eq++;
			*(nbs - ima->data + app->data) = labelnum;
			*nbs = labelnum;
			if(eq>=nxte) eq = nxt;
		}
	}

   }


free((char *) nxt);

rmbrd_int(app);
free_image_int(ima);

}




/*************************************************************************************************************************************************************************
**************************************************************************************************************************************************************************
**************************************************************************************************************************************************************************
**************************************************************************************************************************************************************************/



/* This function create a vector of structure, whose elements are the segments composing the line*/

void compute_diagonal_line_segments_positive_m(int *in,uint16_t *out,int nx, int ny,int i, int j, double m, struct intervals *segments,int linetype, int lineposition){
int L_x = nx/2,L_y=ny/2;
int x,y,x1,y1;
int count = 0,n=0;
int d,count1,k,z;
image_int app,app1;
app=new_image_int(nx,ny);
app1=new_image_int(nx,ny);
int L_d = 2*sqrt(ny*ny + nx*nx);

if(linetype==VERTICAL){
	for(x=0;x<nx-1;x++){
		y= round(j+(x-i)*m);
		y1 = round(j+(x+1-i)*m); 
		if(y>=0 && y<ny && y1>=0 && y1<ny){
			if(lineposition == LEFT){
				if(in[y*nx+x]==in[y1*nx + x+1]){
					if(y1==(ny-1) || (x+1)==nx-1){/*last pixel of the line in the image: the segment ends and i add the Ld*/
						count+=L_d;
						segments[n].lenght = count;
			        		segments[n].lastx = x+1; 
			        		segments[n].lasty = y1; 
						n++;
					}
			 		else {
						if((y==j || x==0)) count +=L_d;/*first pixel of the line in the image: the segment just starts, i all Ld*/
						else count++;
					}
				}
				else{/*first pixel of the segment that is not an extreme segment */	
						if(x==0 || y == j){/*the first pixel is a segment and it is an extreme segment*/
							segments[n].lenght = L_d;
							segments[n].lastx = x; 
							segments[n].lasty = y; 
							count = 0;
							n++;	
						}else if( y1==(ny-1) || (x+1) == (nx-1) ){/*the last pixel is a segment and it is an extreme segment*/
							count++;
							segments[n].lenght = count;
			        			segments[n].lastx = x; 
			        			segments[n].lasty = y; 
							//write the last of each segment
							n++;
							segments[n].lenght = L_d;
							segments[n].lastx = x+1; 
							segments[n].lasty = y1; 
							n++;	
						}else{	
							count++;
							segments[n].lenght = count;
			        			segments[n].lastx = x; 
			        			segments[n].lasty = y; 
							count = 0;
							n++;
						}
				}
			}
			else{/* RIGHT VERTICAL line */
				if(in[y*nx+x]==in[y1*nx + x+1]){
					if(y1==(ny-1) || (x+1)==nx-1){/*last pixel of the line in the image: the segment ends and i add the Ld*/
						count+=L_d;
						segments[n].lenght = count;
			        		segments[n].lastx = x+1; 
			        		segments[n].lasty = y1; 
						n++;
					}
			 		else {
						if((y==0 || x==0)) count +=L_d;/*first pixel of the line in the image: the segment just starts, i all Ld*/
						else count++;
					}
				}
				else{/*first pixel of the segment that is not an extreme segment */	
						if(x==0 || y == 0){/*the first pixel is a segment and it is an extreme segment*/
							segments[n].lenght = L_d;
							segments[n].lastx = x; 
							segments[n].lasty = y; 
							count = 0;
							n++;	
						}else if( y1==(ny-1) || (x+1) == (nx-1) ){/*the last pixel is a segment and it is an extreme segment*/
							count++;
							segments[n].lenght = count;
			        			segments[n].lastx = x; 
			        			segments[n].lasty = y; 
							//write the last of each segment
							n++;
							segments[n].lenght = L_d;
							segments[n].lastx = x+1; 
							segments[n].lasty = y1; 
							n++;	
						}else{	
							count++;
							segments[n].lenght = count;
			        			segments[n].lastx = x; 
			        			segments[n].lasty = y; 
							count = 0;
							n++;
						}
				}
			}
		}
	}

}
else{
/* Line is horizontal */
		for(x=0;x<nx-1;x++){
			y= round(j+(x-i)*m);
			y1 = round(j+(x+1-i)*m); 
			if(y>=0 && y<ny && y1>=0 && y1<ny){
				if(in[y*nx+x]==in[y1*nx + x+1]){
					if(y1==(ny-1) || (x+1)==nx-1){/*last pixel of the line in the image: the segment ends and i add the Ld*/
						count+=L_d;
						segments[n].lenght = count;
			        		segments[n].lastx = x+1; 
			        		segments[n].lasty = y1; 
						n++;
					}
			 		else {
						if((y==0 || x==0)) count +=L_d;/*first pixel of the line in the image: the segment just starts, i all Ld*/
						else count++;
					}
				}
				else{/*first pixel of the segment that is not an extreme segment */	
						if(x==0 || y == 0){/*the first pixel is a segment and it is an extreme segment*/
							segments[n].lenght = L_d;
							segments[n].lastx = x; 
							segments[n].lasty = y; 
							count = 0;
							n++;	
						}else if( y1==(ny-1) || (x+1) == (nx-1) ){/*the last pixel is a segment and it is an extreme segment*/
							count++;
							segments[n].lenght = count;
			        			segments[n].lastx = x; 
			        			segments[n].lasty = y; 
							//write the last of each segment
							n++;
							segments[n].lenght = L_d;
							segments[n].lastx = x+1; 
							segments[n].lasty = y1; 
							n++;	
						}else{	
							count++;
							segments[n].lenght = count;
			        			segments[n].lastx = x; 
			        			segments[n].lasty = y; 
							count = 0;
							n++;
						}
				}
			}
		}

}//end else

		for(d=1;d<n-1;d++){
         		/*initialize app and app1*/
	    		for ( k = 0; k < nx * ny; k++) {
              			app->data[k]=0;app1->data[k]=0;
	    		}
			count=count1=0;
	

			compute_cc(in,app,segments[d-1].lastx,round(j+(segments[d-1].lastx-i)*m),100);
			compute_cc(in,app1,segments[d+1].lastx,round(j+(segments[d+1].lastx-i)*m),200);
			for(x=0; x<nx;x++){
				y= round(j+(x-i)*m);
				if(y>=0 && y<ny){
					if(app->data[y*nx+x]==100 && ((x==0) || y==(ny-1) || y==0 || x==(nx-1)))count+=L_d;
					else if (app->data[y*nx+x]==100)count++;
					if(app1->data[y*nx+x]==200 && ((x==0) || y==(ny-1) || y==0 || x==(nx-1)))count1+=L_d;
					else if(app1->data[y*nx+x]==200) count1++;
				}
			}
			//if(segments[d].lenght<=MIN(segments[d-1].lenght,segments[d+1].lenght)){
			if(segments[d].lenght<=MIN(count,count1)){	
				if(segments[d-1].lastx<segments[d].lastx){
					for(x=segments[d-1].lastx;x<segments[d].lastx;x++){
						y= round(j+(x-i)*m);
						out[y*nx+x] +=1;
					}
				}
				else{
					for(x=segments[d].lastx;x<segments[d-1].lastx;x++){
						y= round(j+(x-i)*m);
						out[y*nx+x] +=1;
					}				    
			       }
			}//end if
		}//end for d


/* Bigger x implies lower regions */
	/*if(n==2){

		if(segments[0].lastx>segments[1].lastx){
			for(x=0;x<segments[0].lastx;x++){
				y= round(j+(x-i)*m);
				out[y*nx+x] +=1;
			}
		}
		else{
			for(x=segments[0].lastx;x<=segments[1].lastx;x++){
				y= round(j+(x-i)*m);
				out[y*nx+x] +=1;
			}
		}
	}*/

           free_image_int(app);
           free_image_int(app1);
}




void compute_diagonal_line_segments_negative_m(int *in,uint16_t *out,int nx, int ny,int i, int j, double m, struct intervals *segments,int linetype, int lineposition){
int L_x = nx/2,L_y=ny/2;
int x,y,x1,y1;
int count = 0,n=0;
int d,count1,k,z;
image_int app,app1;
app=new_image_int(nx,ny);
app1=new_image_int(nx,ny);
int L_d = 2*sqrt(ny*ny + nx*nx);

if(linetype==VERTICAL){
	for(x=0;x<nx-1;x++){
		y= round(j+(x-i)*m);
		y1 = round(j+(x+1-i)*m); 
		if(y>=0 && y<ny && y1>=0 && y1<ny){
			if(lineposition == LEFT){
					if(in[y*nx+x]==in[y1*nx + x+1]){
					if(y1==0 || (x+1)==(nx-1)){/*last pixel of the line in the image: the segment ends and i add the Ld*/
						count+=L_d;
						segments[n].lenght = count;
			        		segments[n].lastx = x+1; 
			        		segments[n].lasty = y1; 
						n++;
					}
			 		else {
						if((y==j || x==0)) count +=L_d;/*first pixel of the line in the image: the segment just starts, i all Ld*/
						else count++;
					}
				}
				else{/*first pixel of the segment that is not an extreme segment */
				
						
						if(x==0 || y ==j){/*the first pixel is a segment and it is an extreme segment*/
							segments[n].lenght = L_d;
							segments[n].lastx = x; 
							segments[n].lasty = y; 
							count = 0;
							n++;	
							}
						else if( y1==0 || (x+1) == (nx-1) ){/*the last pixel is a segment and it is an extreme segment*/

							count++;
							segments[n].lenght = count;
			        			segments[n].lastx = x; 
			        			segments[n].lasty = y; 
							//write the last of each segment
							n++;
							segments[n].lenght = L_d;
							segments[n].lastx = x+1; 
							segments[n].lasty = y1; 
							n++;	
							}
						else{	
							count++;
							segments[n].lenght = count;
			        			segments[n].lastx = x; 
			        			segments[n].lasty = y; 
							count = 0;
							n++;
						}
					
			
				}
			}
			else{/* RIGHT VERTICAL line */
				if(in[y*nx+x]==in[y1*nx + x+1]){
					if(y1==(ny-1) || (x+1)==nx-1){/*last pixel of the line in the image: the segment ends and i add the Ld*/
						count+=L_d;
						segments[n].lenght = count;
			        		segments[n].lastx = x+1; 
			        		segments[n].lasty = y1; 
						n++;
					}
			 		else {
						if((y==0 || x==0)) count +=L_d;/*first pixel of the line in the image: the segment just starts, i all Ld*/
						else count++;
					}
				}
				else{/*first pixel of the segment that is not an extreme segment */	
						if(x==0 || y == 0){/*the first pixel is a segment and it is an extreme segment*/
							segments[n].lenght = L_d;
							segments[n].lastx = x; 
							segments[n].lasty = y; 
							count = 0;
							n++;	
						}else if( y1==(ny-1) || (x+1) == (nx-1) ){/*the last pixel is a segment and it is an extreme segment*/
							count++;
							segments[n].lenght = count;
			        			segments[n].lastx = x; 
			        			segments[n].lasty = y; 
							//write the last of each segment
							n++;
							segments[n].lenght = L_d;
							segments[n].lastx = x+1; 
							segments[n].lasty = y1; 
							n++;	
						}else{	
							count++;
							segments[n].lenght = count;
			        			segments[n].lastx = x; 
			        			segments[n].lasty = y; 
							count = 0;
							n++;
						}
				}
			}
		}
	}

}
else{
/* Line is horizontal */
		for(x=0;x<nx-1;x++){
			y= round(j+(x-i)*m);
			y1 = round(j+(x+1-i)*m); 
			if(y>=0 && y<ny && y1>=0 && y1<ny){
				if(in[y*nx+x]==in[y1*nx + x+1]){
					if(y1==0 || (x+1)==nx-1){/*last pixel of the line in the image: the segment ends and i add the Ld*/
						count+=L_d;
						segments[n].lenght = count;
			        		segments[n].lastx = x+1; 
			        		segments[n].lasty = y1; 
						n++;
					}
			 		else {
						if((y==ny-1 || x==0)) count +=L_d;/*first pixel of the line in the image: the segment just starts, i all Ld*/
						else count++;
					}
				}
				else{/*first pixel of the segment that is not an extreme segment */
				
						
						if(x==0 || y == (ny-1)){/*the first pixel is a segment and it is an extreme segment*/
							segments[n].lenght = L_d;
							segments[n].lastx = x; 
							segments[n].lasty = y; 
							count = 0;
							n++;	
							}
						else if( y1==0 || (x+1) == (nx-1) ){/*the last pixel is a segment and it is an extreme segment*/

							count++;
							segments[n].lenght = count;
			        			segments[n].lastx = x; 
			        			segments[n].lasty = y; 
							//write the last of each segment
							n++;
							segments[n].lenght = L_d;
							segments[n].lastx = x+1; 
							segments[n].lasty = y1; 
							n++;	
							}
						else{	
							count++;
							segments[n].lenght = count;
			        			segments[n].lastx = x; 
			        			segments[n].lasty = y; 
							count = 0;
							n++;
						}
					
			
				}
			}
		}

		
}//end else


	for(d=1;d<n-1;d++){
         		/*initialize app and app1*/
	    		for ( k = 0; k < nx * ny; k++) {
              			app->data[k]=0;app1->data[k]=0;
	    		}
			count=count1=0;
	

			compute_cc(in,app,segments[d-1].lastx,round(j+(segments[d-1].lastx-i)*m),100);
			compute_cc(in,app1,segments[d+1].lastx,round(j+(segments[d+1].lastx-i)*m),200);
			for(x=0; x<nx;x++){
				y= round(j+(x-i)*m);
				if(y>=0 && y<ny){
					if(app->data[y*nx+x]==100 && ((x==0) || y==(ny-1) || y==0 || x==(nx-1)))count+=L_d;
					else if (app->data[y*nx+x]==100)count++;
					if(app1->data[y*nx+x]==200 && ((x==0) || y==(ny-1) || y==0 || x==(nx-1)))count1+=L_d;
					else if(app1->data[y*nx+x]==200) count1++;
				}
			}
			//if(segments[d].lenght<=MIN(segments[d-1].lenght,segments[d+1].lenght)){
			if(segments[d].lenght<=MIN(count,count1)){
				if(segments[d-1].lastx<segments[d].lastx){
					for(x=segments[d-1].lastx;x<segments[d].lastx;x++){
						y= round(j+(x-i)*m);
						out[y*nx+x] +=1;
					}
				}
				else{
					for(x=segments[d].lastx;x<segments[d-1].lastx;x++){
						y= round(j+(x-i)*m);
						out[y*nx+x] +=1;
					}				    
			       }
			}//end if
		}//end for d
/* Smaller x implies lower regions */
	/*if(n==2){	
		if(segments[0].lastx<segments[1].lastx){
			for(x=i;x<segments[0].lastx;x++){
				y= round(j+(x-i)*m);
				//printf(" %d %d %d \n",x,y, nx);
				out[y*nx+x] +=1;
			}
		}
	}*/

           free_image_int(app);
           free_image_int(app1);
}

void compute_horizontal_line_segments(int *in,uint16_t *out,int nx, int ny,int j, struct intervals *segments){

int k,d,count,count1,first,n;
image_int app,app1;
app=new_image_int(nx,ny);
app1=new_image_int(nx,ny);
int L_x = 2*nx;


/* (i,j) are the coordinate of a point in the upper vertical line: (0,j) */
		/* horizontal */
		count = n = first =0;
		for(k=0;k<nx-1;k++){
			if(in[k+j*nx]==in[k+1+j*nx]){
				if((k+1)==(nx-1)){
					count+=L_x;
					segments[n].lenght = count;
					segments[n].lastx = k+1; 
					n++;
				}
			 	else {
					if(k==0) count +=L_x;
					else count++;
					}
			}
			else{
				if(k==0 ){
					count +=L_x;
					segments[n].lenght = count;
					segments[n].lastx = k+1; 
					//write the last of each segment
					count = 0;
					n++;	
					}
				else if((k+1)==(nx-1)){
					count++;
					segments[n].lenght = count;
					segments[n].lastx = k+1; 
					//write the last of each segment
					n++;
					segments[n].lenght = L_x;
					segments[n].lastx = k+1; 
					n++;	
					}
				else{
					count++;
					segments[n].lenght= count;
					segments[n].lastx=k+1;
					n++;
					count=0;

				}
			}
		}//end for k

		for(d=1;d<n-1;d++){
		// Compute composante conexe à laquelle appartient le première interval 
         		/*initialize app and app1*/
	   		 for ( k = 0; k < nx * ny; k++) {
              			app->data[k]=0;app1->data[k]=0;
	    		}
		
			count=count1=0;
			compute_cc(in,app,segments[d-1].lastx-1,j,100);
			compute_cc(in,app1,segments[d+1].lastx-1,j,200);
			for(k=0;k<nx;k++){
				if(app->data[j*nx+k]==100 && (k==0 || k==(ny-1)))count+=L_x;
				else if (app->data[j*nx+k]==100)count++;
				if(app1->data[j*nx+k]==200 && ( k==0 || k==(ny-1)))count1+=L_x;
				else if(app1->data[j*nx+k]==200) count1++;				
			}
			//if(segments[d].lenght<=MIN(segments[d-1].lenght,segments[d+1].lenght)){
			if(segments[d].lenght<=MIN(count,count1)){
				for(k=segments[d-1].lastx;k<segments[d].lastx;k++)out[j*nx+k] +=1;
			
			}			
		
		}


           free_image_int(app);
           free_image_int(app1);
}

void compute_vertical_line_segments(int *in,uint16_t *out,int nx, int ny,int i, struct intervals *segments){
int k,d,count,count1,first,n;
image_int app,app1;
app=new_image_int(nx,ny);
app1=new_image_int(nx,ny);
int L_y= 2*ny;

/* (i,j) are the coordinate of a point in the upper horizontal line: (i,0) */
		/* vertical */
		count = n = first =0;
		for(k=0;k<ny-1;k++){	
			if(in[i+k*nx]==in[i+(k+1)*nx]){
				if((k+1)==(ny-1)){
					count+=L_y;
					segments[n].lenght = count;
					segments[n].lasty = k+1;
					n++;
				}
			 	else {
					if(k==0) count +=L_y;
					else count++;
					}
			}
			else{

				if(k==0 ){
					count +=L_y;
					segments[n].lenght = count;
					segments[n].lasty = k+1; 
					//write the last of each segment
					count = 0;
					n++;	
					}
				else if((k+1)==(ny-1)){
					count++;
					segments[n].lenght = count;
					segments[n].lasty = k+1; 
					//write the last of each segment
					n++;
					segments[n].lenght = L_y;
					segments[n].lasty = k+1; 
					n++;	
					}
				else{
					count++;
					segments[n].lenght = count;
					segments[n].lasty = k+1; 		
					count = 0;
					n++;
				}
			}
		}
		for(d=1;d<n-1;d++){
         		/*initialize app and app1*/
	   		 for ( k = 0; k < nx * ny; k++) {
              			app->data[k]=0;app1->data[k]=0;
	    		}
			count=count1=0;
			compute_cc(in,app,i,segments[d-1].lasty-1,100);
			compute_cc(in,app1,i,segments[d+1].lasty-1,200);
			for(k=0;k<ny;k++){	
				if(app->data[k*nx+i]==100 && (k==0 || k==(ny-1)))count+=L_y;
				else if (app->data[k*nx+i]==100)count++;
				if(app1->data[k*nx+i]==200 && (k==0 || k==(ny-1)))count1+=L_y;
				else if(app1->data[k*nx+i]==200) count1++;

			}
			
			//if(segments[d].lenght<=MIN(segments[d-1].lenght,segments[d+1].lenght)){
			if(segments[d].lenght<=MIN(count,count1)){
				for(k=segments[d-1].lasty;k<segments[d].lasty;k++)out[k*nx+i] +=1;
			
			}
		}
	/* check if there are only two segments */
		if(n==2){
			if(segments[0].lasty>segments[1].lasty)
				for(k=0;k<segments[0].lasty;k++)out[k*nx+i] +=1;
			else 
				for(k=segments[0].lasty;k<=segments[1].lasty;k++)out[k*nx+i] +=1;
		}

           free_image_int(app);
           free_image_int(app1);
        

}


void trace_line(int *in,int nx,int ny,struct intervals *segments,uint16_t *out, int linetype, int ini, int bound,int lineposition){
int i,j,k;
double theta,m,x,y;
if(linetype == VERTICAL){/* vertical line of the perimeter*/
	i=ini;
	for(j=0;j<bound;j++){
			for(k=0;k<N_LINES;k++){
				theta = (k*M_PI)/N_LINES;
				if(theta!=M_PI/2){/* do not consider the vertical line through the point (i,j)*/
					if(theta == 0.0) 
						compute_horizontal_line_segments(in,out,nx,ny,j,segments); /* horizontal line through the point (i,j) */
					else{
						m = sin(theta)/cos(theta);
					/* the line_segments function constructs the vector whose elements are segments composing the intersection of the line with the image*/
						
						if(m<0.)compute_diagonal_line_segments_negative_m(in,out,nx,ny,i,j,m,segments,linetype,lineposition);
						else  compute_diagonal_line_segments_positive_m(in,out,nx,ny,i,j,m,segments,linetype,lineposition);
						
					}
									
				}		
			}	
	}
}else{ 
	j=ini;
	for(i=0;i<bound;i++){ 
/* this function consider a line of the perimeter of the image */
/* horizontal line of the perimeter: do not consider the horizontal lines *//* 0 < j < ny */
		for(k=1;k<N_LINES;k++){/* do not consider the horizontal line through the point (i,j) -> k starts with value 1 */
			theta = (k*M_PI)/N_LINES;
			if(theta!=M_PI/2){
				m = sin(theta)/cos(theta);
				
				if(m<0.)compute_diagonal_line_segments_negative_m(in,out,nx,ny,i,j,m,segments,linetype,lineposition);
				else compute_diagonal_line_segments_positive_m(in,out,nx,ny,i,j,m,segments,linetype,lineposition);
							
			}
			else compute_vertical_line_segments(in,out,nx,ny,i,segments);
		}
	}//end for
}//end else

}

/* The input is a binary image, the output is an estimation of relative depth of the binary image*/
void generalized_convexity(int *in,int nx,int ny, uint16_t *out){
struct intervals *segments;
/* vector storing the segment length */
int lenght = nx + ny;
segments = (struct intervals *)malloc((unsigned int)lenght*sizeof(struct intervals)+1);

/* make a binary image */

/* upper horizontal line of the image perimeter */
trace_line(in,nx,ny,segments,out,HORIZONTAL,0,nx,UP);

/* left vertical line of the image perimeter */
trace_line(in,nx,ny,segments,out,VERTICAL,0,ny,LEFT);

/* lower horizontal line of the image perimeter */
trace_line(in,nx,ny,segments,out,HORIZONTAL,ny-1,nx,DOWN);

/* rigth vertical line of the image perimeter */
trace_line(in,nx,ny,segments,out,VERTICAL,nx-1,ny,RIGHT);

free(segments);
}


int main(int argc, char *argv[])
{
    if(argc < 3)
        printf("image out: ");
    else
    {
        //read the parameters
        int i=1;
        const char* image  = argv[i]; i++;
        const char* outfile = argv[i]; i++;

	int *im, nx, ny;
        image_float  imoutfloat;
	uint16_t *imout;
	float min, max;
        char fname[100];
	
        if(read_image_int(image, &im, &nx, &ny) != -1)
        {
            //allocate memory for the flow
            imout = (uint16_t *) calloc( (size_t) (nx*ny), sizeof(uint16_t) );
	    for (i = 0; i < nx * ny; i++) imout[i]=0;
	    
		
            generalized_convexity(im, nx, ny,imout);         
	    imoutfloat = new_image_float(nx,ny);
	    for ( i = 0; i < nx * ny; i++) 
              imoutfloat->data[i]=(float)imout[i];
	    
      
            
            sprintf(fname, "%s.pgm", outfile);
	    iio_save_image_float(fname,imoutfloat->data,imoutfloat->xsize,imoutfloat->ysize);

            free_image_float(imoutfloat);
            free(imout);
	    free((int *)im);
      
        }
        else  fatal_error("Cannot read the images");
        
    }

    return 0;
}


/*

1. create binary images 
2. for each binary image: 
	for each pixel of the perimeter
		a. consider n directions
		b. for each direction: consider the intersection of the line having the considered direction with the binary image
		c. compute the number of intervals composing the intersection
		d. for each interval (except the first and the last), assign depth to all point of the interval (or to all point of the connected component the segment belong to)


*/

